@@ -1,8 +1,25 @@
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
 
 """Interface documentation.
 
 Define the interfaces that are implemented by various buildbot classes.
 """
+# E0211: Method has no argument
+# E0213: Method should have "self" as first argument
+# pylint: disable-msg=E0211,E0213
 
 from zope.interface import Interface, Attribute
 
@@ -20,77 +37,24 @@ class LatentBuildSlaveFailedToSubstantiate(Exception):
 class BuildbotNotRunningError(Exception):
     pass
 
+class ParameterError(Exception):
+    pass
+
 class IChangeSource(Interface):
-    """Object which feeds Change objects to the changemaster. When files or
-    directories are changed and the version control system provides some
-    kind of notification, this object should turn it into a Change object
-    and pass it through::
-
-      self.changemaster.addChange(change)
     """
+    Service which feeds Change objects to the changemaster. When files or
+    directories are changed in version control, this object should represent
+    the changes as a change dictionary and call::
 
-    def start():
-        """Called when the buildmaster starts. Can be used to establish
-        connections to VC daemons or begin polling."""
+      self.master.addChange(who=.., rev=.., ..)
 
-    def stop():
-        """Called when the buildmaster shuts down. Connections should be
-        terminated, polling timers should be canceled."""
+    See 'Writing Change Sources' in the manual for more information.
+    """
+    master = Attribute('master',
+            'Pointer to BuildMaster, automatically set when started.')
 
     def describe():
-        """Should return a string which briefly describes this source. This
-        string will be displayed in an HTML status page."""
-
-class IScheduler(Interface):
-    """I watch for Changes in the source tree and decide when to trigger
-    Builds. I create BuildSet objects and submit them to the BuildMaster. I
-    am a service, and the BuildMaster is always my parent.
-
-    @ivar properties: properties to be applied to all builds started by this
-    scheduler
-    @type properties: L<buildbot.process.properties.Properties>
-    """
-
-    def addChange(change):
-        """A Change has just been dispatched by one of the ChangeSources.
-        Each Scheduler will receive this Change. I may decide to start a
-        build as a result, or I might choose to ignore it."""
-
-    def listBuilderNames():
-        """Return a list of strings indicating the Builders that this
-        Scheduler might feed."""
-
-    def getPendingBuildTimes():
-        """Return a list of timestamps for any builds that are waiting in the
-        tree-stable-timer queue. This is only relevant for Change-based
-        schedulers, all others can just return an empty list."""
-        # TODO: it might be nice to make this into getPendingBuildSets, which
-        # would let someone subscribe to the buildset being finished.
-        # However, the Scheduler doesn't actually create the buildset until
-        # it gets submitted, so doing this would require some major rework.
-
-class IUpstreamScheduler(Interface):
-    """This marks an IScheduler as being eligible for use as the 'upstream='
-    argument to a buildbot.scheduler.Dependent instance."""
-
-    def subscribeToSuccessfulBuilds(target):
-        """Request that the target callbable be invoked after every
-        successful buildset. The target will be called with a single
-        argument: the SourceStamp used by the successful builds."""
-
-    def listBuilderNames():
-        """Return a list of strings indicating the Builders that this
-        Scheduler might feed."""
-
-class IDownstreamScheduler(Interface):
-    """This marks an IScheduler to be listening to other schedulers.
-    On reconfigs, these might get notified to check if their upstream
-    scheduler are stil the same."""
-
-    def checkUpstreamScheduler():
-        """Check if the upstream scheduler is still alive, and if not,
-        get a new upstream object from the master."""
-
+        """Return a string which briefly describes this source."""
 
 class ISourceStamp(Interface):
     """
@@ -103,10 +67,16 @@ class ISourceStamp(Interface):
     @cvar patch: patch applied to the source, or None if no patch
     @type patch: None or tuple (level diff)
 
-    @cvar changes: the source step should check out hte latest revision
+    @cvar changes: the source step should check out the latest revision
                    in the given changes
     @type changes: tuple of L{buildbot.changes.changes.Change} instances,
                    all of which are on the same branch
+
+    @cvar project: project this source code represents
+    @type project: string
+
+    @cvar repository: repository from which source was drawn
+    @type repository: string
     """
 
     def canBeMergedWith(self, other):
@@ -116,7 +86,7 @@ class ISourceStamp(Interface):
 
     def mergeWith(self, others):
         """Generate a SourceStamp for the merger of me and all the other
-        BuildRequests. This is called by a Build when it starts, to figure
+        SourceStamps. This is called by a Build when it starts, to figure
         out what its sourceStamp should be."""
 
     def getAbsoluteSourceStamp(self, got_revision):
@@ -144,10 +114,10 @@ class IStatus(Interface):
     """I am an object, obtainable from the buildmaster, which can provide
     status information."""
 
-    def getProjectName():
+    def getTitle():
         """Return the name of the project that this Buildbot is working
         for."""
-    def getProjectURL():
+    def getTitleURL():
         """Return the URL of this Buildbot's project."""
     def getBuildbotURL():
         """Return the URL of the top-most Buildbot status page, or None if
@@ -181,7 +151,11 @@ class IStatus(Interface):
         """Return the ISlaveStatus object for a given named buildslave."""
 
     def getBuildSets():
-        """Return a list of active (non-finished) IBuildSetStatus objects."""
+        """
+        Return a list of un-completed build sets.
+
+        @returns: list of L{IBuildSetStatus} implementations, via Deferred.
+        """
 
     def generateFinishedBuilds(builders=[], branches=[],
                                num_builds=None, finished_before=None,
@@ -236,13 +210,6 @@ class IBuildSetStatus(Interface):
     """I represent a set of Builds, each run on a separate Builder but all
     using the same source tree."""
 
-    def getSourceStamp():
-        """Return a SourceStamp object which can be used to re-create
-        the source tree that this build used.
-
-        This method will return None if the source information is no longer
-        available."""
-        pass
     def getReason():
         pass
     def getID():
@@ -255,12 +222,9 @@ class IBuildSetStatus(Interface):
         pass # not implemented
     def getBuilderNames():
         """Return a list of the names of all Builders on which this set will
-        do builds."""
-    def getBuildRequests():
-        """Return a list of IBuildRequestStatus objects that represent my
-        component Builds. This list might correspond to the Builders named by
-        getBuilderNames(), but if builder categories are used, or 'Builder
-        Aliases' are implemented, then they may not."""
+        do builds.
+        
+        @returns: list of names via Deferred"""
     def isFinished():
         pass
     def waitUntilSuccess():
@@ -271,7 +235,9 @@ class IBuildSetStatus(Interface):
         """Return a Deferred that fires (with this IBuildSetStatus object)
         when all builds have finished."""
     def getResults():
-        pass
+        """Return SUCCESS/FAILURE, or None if the buildset is not finished
+        yet"""
+
 
 class IBuildRequestStatus(Interface):
     """I represent a request to build a particular set of source code on a
@@ -279,18 +245,19 @@ class IBuildRequestStatus(Interface):
     finally turned into a Build."""
 
     def getSourceStamp():
-        """Return a SourceStamp object which can be used to re-create
-        the source tree that this build used.  This method will
-        return an absolute SourceStamp if possible, and its results
-        may change as the build progresses.  Specifically, a "HEAD"
-        build may later be more accurately specified by an absolute
-        SourceStamp with the specific revision information.
+        """
+        Get a SourceStamp object which can be used to re-create the source tree
+        that this build used.  This method will return an absolute SourceStamp
+        if possible, and its results may change as the build progresses.
+        Specifically, a "HEAD" build may later be more accurately specified by
+        an absolute SourceStamp with the specific revision information.
 
         This method will return None if the source information is no longer
-        available."""
-        pass
-    def getBuilderName():
-        pass
+        available.
+
+        @returns: SourceStamp via Deferred
+        """
+
     def getBuilds():
         """Return a list of IBuildStatus objects for each Build that has been
         started in an attempt to satify this BuildRequest."""
@@ -306,9 +273,8 @@ class IBuildRequestStatus(Interface):
     def unsubscribe(observer):
         """Unregister the callable that was registered with subscribe()."""
     def getSubmitTime():
-        """Return the time when this request was submitted"""
-    def setSubmitTime(t):
-        """Sets the time when this request was submitted"""
+        """Return the time when this request was submitted.  Returns a
+        Deferred."""
 
 
 class ISlaveStatus(Interface):
@@ -357,10 +323,13 @@ class IBuilderStatus(Interface):
         """Return a list of ISlaveStatus objects for the buildslaves that are
         used by this builder."""
 
-    def getPendingBuilds():
-        """Return an IBuildRequestStatus object for all upcoming builds
-        (those which are ready to go but which are waiting for a buildslave
-        to be available."""
+    def getPendingBuildRequestStatuses():
+        """
+        Get a L{IBuildRequestStatus} implementations for all unclaimed build
+        requests.
+
+        @returns: list of objects via Deferred
+        """
 
     def getCurrentBuilds():
         """Return a list containing an IBuildStatus object for each build
@@ -443,7 +412,7 @@ class IBuilderStatus(Interface):
         delivered."""
 
 class IEventSource(Interface):
-    def eventGenerator(branches=[], categories=[]):
+    def eventGenerator(branches=[], categories=[], committers=[], minTime=0):
         """This function creates a generator which will yield all of this
         object's status events, starting with the most recent and progressing
         backwards in time. These events provide the IStatusEvent interface.
@@ -459,6 +428,13 @@ class IEventSource(Interface):
         should only return events that are categorized within the
         given category.  If the list is empty, events for all
         categories should be returned.
+
+        @param comitters: a list of committers.  The generator should only
+        return events caused by one of the listed committers. If the list is
+        empty or None, events from every committers should be returned.
+
+        @param minTime: a timestamp. Do not generate events occuring prior to
+        this timestamp.
         """
 
 class IBuildStatus(Interface):
@@ -481,10 +457,6 @@ class IBuildStatus(Interface):
         build has already finished, this deferred will fire right away. The
         callback is given this IBuildStatus instance as an argument."""
 
-    def getProperty(propname):
-        """Return the value of the build property with the given name. Raises
-        KeyError if there is no such property on this build."""
-
     def getReason():
         """Return a string that indicates why the build was run. 'changes',
         'forced', and 'periodic' are the most likely values. 'try' will be
@@ -503,6 +475,10 @@ class IBuildStatus(Interface):
         """Return a list of Change objects which represent which source
         changes went into the build."""
 
+    def getRevisions():
+        """Returns a string representing the list of revisions that led to
+        the build, rendered from each Change.revision"""
+
     def getResponsibleUsers():
         """Return a list of Users who are to blame for the changes that went
         into this build. If anything breaks (at least anything that wasn't
@@ -932,6 +908,10 @@ class IStatusReceiver(Interface):
         """This is a periodic update on the progress this Build has made
         towards completion."""
 
+    def changeAdded(change):
+        """A new Change was added to the ChangeMaster.  By the time this event
+        is received, all schedulers have already received the change."""
+
     def stepStarted(build, step):
         """A step has just started. 'step' is the IBuildStepStatus which
         represents the step: it can be queried for more information.
@@ -993,49 +973,55 @@ class IStatusReceiver(Interface):
         in L{IBuildStatus.getResults}.
 
         @type  builderName: string
-        @type  build:       L{buildbot.status.builder.BuildStatus}
+        @type  build:       L{buildbot.status.build.BuildStatus}
         @type  results:     tuple
         """
 
     def builderRemoved(builderName):
         """The Builder has been removed."""
 
+    def slaveConnected(slaveName):
+        """The slave has connected."""
+
+    def slaveDisconnected(slaveName):
+        """The slave has disconnected."""
+
+    def checkConfig(otherStatusReceivers):
+        """Verify that there are no other status receivers which conflict with
+        the current one.
+
+        @type  otherStatusReceivers: A list of L{IStatusReceiver} objects which
+        will contain self.
+        """
+
+
 class IControl(Interface):
     def addChange(change):
-        """Add a change to all builders. Each Builder will decide for
-        themselves whether the change is interesting or not, and may initiate
-        a build as a result."""
-
-    def submitBuildSet(buildset):
-        """Submit a BuildSet object, which will eventually be run on all of
-        the builders listed therein."""
+        """Add a change to the change queue, for analysis by schedulers."""
 
     def getBuilder(name):
         """Retrieve the IBuilderControl object for the given Builder."""
 
 class IBuilderControl(Interface):
-    def requestBuild(request):
-        """Queue a L{buildbot.process.base.BuildRequest} object for later
-        building."""
+    def submitBuildRequest(ss, reason, props=None):
+        """Create a BuildRequest, which will eventually cause a build of the
+        given SourceStamp to be run on this builder. This returns a
+        BuildRequestStatus object via a Deferred, which can be used to keep
+        track of the builds that are performed."""
 
-    def requestBuildSoon(request):
-        """Submit a BuildRequest like requestBuild, but raise a
-        L{buildbot.interfaces.NoSlaveError} if no slaves are currently
-        available, so it cannot be used to queue a BuildRequest in the hopes
-        that a slave will eventually connect. This method is appropriate for
-        use by things like the web-page 'Force Build' button."""
-
-    def resubmitBuild(buildStatus, reason="<rebuild, no reason given>"):
+    def rebuildBuild(buildStatus, reason="<rebuild, no reason given>"):
         """Rebuild something we've already built before. This submits a
         BuildRequest to our Builder using the same SourceStamp as the earlier
         build. This has no effect (but may eventually raise an exception) if
         this Build has not yet finished."""
 
-    def getPendingBuilds():
-        """Return a list of L{IBuildRequestControl} objects for this Builder.
-        Each one corresponds to a pending build that has not yet started (due
-        to a scarcity of build slaves). These upcoming builds can be canceled
-        through the control object."""
+    def getPendingBuildRequestControls():
+        """
+        Get a list of L{IBuildRequestControl} objects for this Builder.
+        Each one corresponds to an unclaimed build request.
+
+        @returns: list of objects via Deferred
+        """
 
     def getBuild(number):
         """Attempt to return an IBuildControl object for the given build.
@@ -1127,13 +1113,97 @@ class ILatentBuildSlave(IBuildSlave):
     def buildStarted(sb):
         """Inform the latent build slave that a build has started.
 
-        ``sb`` is a LatentSlaveBuilder as defined in buildslave.py.  The sb
-        is the one for whom the build started.
+        @param sb: a L{LatentSlaveBuilder}.  The sb is the one for whom the
+        build finished.
         """
 
     def buildFinished(sb):
         """Inform the latent build slave that a build has finished.
 
-        ``sb`` is a LatentSlaveBuilder as defined in buildslave.py.  The sb
-        is the one for whom the build finished.
+        @param sb: a L{LatentSlaveBuilder}.  The sb is the one for whom the
+        build finished.
+        """
+
+class IRenderable(Interface):
+    """An object that can be interpolated with properties from a build.
+    """
+
+    def getRenderingFor(iprops):
+        """Return the interpolation with the given properties
+
+        @param iprops: the L{IProperties} provider supplying the properties.
+        """
+class IProperties(Interface):
+    """
+    An object providing access to build properties
+    """
+
+    def getProperty(name, default=None):
+        """Get the named property, returning the default if the property does
+        not exist.
+
+        @param name: property name
+        @type name: string
+
+        @param default: default value (default: @code{None})
+
+        @returns: property value
+        """
+
+    def hasProperty(name):
+        """Return true if the named property exists.
+
+        @param name: property name
+        @type name: string
+        @returns: boolean
+        """
+
+    def has_key(name):
+        """Deprecated name for L{hasProperty}."""
+
+    def setProperty(name, value, source, runtime=False):
+        """Set the given property, overwriting any existing value.  The source
+        describes the source of the value for human interpretation.
+
+        @param name: property name
+        @type name: string
+
+        @param value: property value
+        @type value: JSON-able value
+
+        @param source: property source
+        @type source: string
+
+        @param runtime: (optional) whether this property was set during the
+        build's runtime: usually left at its default value
+        @type runtime: boolean
+        """
+
+    def getProperties():
+        """Get the L{buildbot.process.properties.Properties} instance storing
+        these properties.  Note that the interface for this class is not
+        stable, so where possible the other methods of this interface should be
+        used.
+
+        @returns: L{buildbot.process.properties.Properties} instance
+        """
+
+    def getBuild():
+        """Get the L{buildbot.process.build.Build} instance for the current
+        build.  Note that this object is not available after the build is
+        complete, at which point this method will return None.
+
+        Try to avoid using this method, as the API of L{Build} instances is not
+        well-defined.
+
+        @returns L{buildbot.process.build.Build} instance
+        """
+
+    def render(value):
+        """Render @code{value} as an L{IRenderable}.  This essentially coerces
+        @code{value} to an L{IRenderable} and calls its @L{getRenderingFor}
+        method.
+
+        @name value: value to render
+        @returns: rendered value
         """
