@@ -1,10 +1,25 @@
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 
 import os, sys, time
 
 class Follower:
     def follow(self):
         from twisted.internet import reactor
-        from buildbot.scripts.reconfig import LogWatcher
+        from buildbot.scripts.logwatcher import LogWatcher
         self.rc = 0
         print "Following twistd.log until startup finished.."
         lw = LogWatcher("twistd.log")
@@ -102,7 +117,6 @@ def launch(config):
         # see if we can launch the application without actually having to
         # spawn twistd, since spawning processes correctly is a real hassle
         # on windows.
-        from twisted.python.runtime import platformType
         argv = ["twistd",
                 "--no_save",
                 "--logfile=twistd.log", # windows doesn't use the same default
@@ -112,15 +126,6 @@ def launch(config):
         # this is copied from bin/twistd. twisted-2.0.0 through 2.4.0 use
         # _twistw.run . Twisted-2.5.0 and later use twistd.run, even for
         # windows.
-        from twisted import __version__
-        major, minor, ignored = __version__.split(".", 2)
-        major = int(major)
-        minor = int(minor)
-        if (platformType == "win32" and (major == 2 and minor < 5)):
-            from twisted.scripts import _twistw
-            run = _twistw.run
-        else:
-            from twisted.scripts import twistd
-            run = twistd.run
-        run()
+        from twisted.scripts import twistd
+        twistd.run()
 
