@@ -18,10 +18,12 @@ import urlparse, urllib, time, re
 import os, cgi, sys, locale
 import jinja2
 from zope.interface import Interface
-from twisted.web import resource, static
+from twisted.internet import defer
+from twisted.web import resource, static, server
 from twisted.python import log
-from buildbot.status import builder
-from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, SKIPPED, EXCEPTION, RETRY
+from buildbot.status import builder, buildstep, build
+from buildbot.status.results import SUCCESS, WARNINGS, FAILURE, SKIPPED
+from buildbot.status.results import EXCEPTION, RETRY
 from buildbot import version, util
 from buildbot.process.properties import Properties
 
@@ -60,11 +62,14 @@ css_classes = {SUCCESS: "success",
 
 def getAndCheckProperties(req):
     """
-Fetch custom build properties from the HTTP request of a "Force build" or
-"Resubmit build" HTML form.
-Check the names for valid strings, and return None if a problem is found.
-Return a new Properties object containing each property found in req.
-"""
+    Fetch custom build properties from the HTTP request of a "Force build" or
+    "Resubmit build" HTML form.
+    Check the names for valid strings, and return None if a problem is found.
+    Return a new Properties object containing each property found in req.
+    """
+    master = req.site.buildbot_service.master
+    pname_validate = master.config.validation['property_name']
+    pval_validate = master.config.validation['property_value']
     properties = Properties()
     i = 1
     while True:
@@ -72,8 +77,8 @@ Return a new Properties object containing each property found in req.
         pvalue = req.args.get("property%dvalue" % i, [""])[0]
         if not pname:
             break
-        if not re.match(r'^[\w\.\-\/\~:]*$', pname) \
-                or not re.match(r'^[\w\.\-\/\~:]*$', pvalue):
+        if not pname_validate.match(pname) \
+                or not pval_validate.match(pvalue):
             log.msg("bad property name='%s', value='%s'" % (pname, pvalue))
             return None
         properties.setProperty(pname, pvalue, "Force Build Form")
@@ -88,10 +93,9 @@ def build_get_class(b):
     """
     # FIXME: this getResults duplicity might need to be fixed
     result = b.getResults()
-    #print "THOMAS: result for b %r: %r" % (b, result)
-    if isinstance(b, builder.BuildStatus):
+    if isinstance(b, build.BuildStatus):
         result = b.getResults()
-    elif isinstance(b, builder.BuildStepStatus):
+    elif isinstance(b, buildstep.BuildStepStatus):
         result = b.getResults()[0]
         # after forcing a build, b.getResults() returns ((None, []), []), ugh
         if isinstance(result, tuple):
@@ -170,34 +174,12 @@ class Box:
         return props    
     
     
-class ContextMixin(object):
-    def getContext(self, request):
-        status = self.getStatus(request)
-        rootpath = path_to_root(request)
-        locale_enc = locale.getdefaultlocale()[1]
-        if locale_enc is not None:
-            locale_tz = unicode(time.tzname[time.localtime()[-1]], locale_enc)
-        else:
-            locale_tz = unicode(time.tzname[time.localtime()[-1]])
-        return dict(project_url = status.getProjectURL(),
-                    project_name = status.getProjectName(),
-                    stylesheet = rootpath + 'default.css',
-                    path_to_root = rootpath,
-                    version = version,
-                    time = time.strftime("%a %d %b %Y %H:%M:%S",
-                                        time.localtime(util.now())),
-                    tz = locale_tz,
-                    metatags = [],
-                    title = self.getTitle(request),
-                    welcomeurl = rootpath,
-                    authz = self.getAuthz(request),
-                    )
-
+class AccessorMixin(object):
     def getStatus(self, request):
         return request.site.buildbot_service.getStatus()    
         
-    def getTitle(self, request):
-        return self.title
+    def getPageTitle(self, request):
+        return self.pageTitle
 
     def getAuthz(self, request):
         return request.site.buildbot_service.authz
@@ -206,10 +188,69 @@ class ContextMixin(object):
         return request.site.buildbot_service.master
 
 
+class ContextMixin(AccessorMixin):
+    def getContext(self, request):
+        status = self.getStatus(request)
+        rootpath = path_to_root(request)
+        locale_enc = locale.getdefaultlocale()[1]
+        if locale_enc is not None:
+            locale_tz = unicode(time.tzname[time.localtime()[-1]], locale_enc)
+        else:
+            locale_tz = unicode(time.tzname[time.localtime()[-1]])
+        return dict(title_url = status.getTitleURL(),
+                    title = status.getTitle(),
+                    stylesheet = rootpath + 'default.css',
+                    path_to_root = rootpath,
+                    version = version,
+                    time = time.strftime("%a %d %b %Y %H:%M:%S",
+                                        time.localtime(util.now())),
+                    tz = locale_tz,
+                    metatags = [],
+                    pageTitle = self.getPageTitle(request),
+                    welcomeurl = rootpath,
+                    authz = self.getAuthz(request),
+                    )
+
+
+class ActionResource(resource.Resource, AccessorMixin):
+    """A resource that performs some action, then redirects to a new URL."""
+
+    isLeaf = 1
+
+    def getChild(self, name, request):
+        return self
+
+    def performAction(self, request):
+        """
+        Perform the action, and return the URL to redirect to
+
+        @param request: the web request
+        @returns: URL via Deferred
+        """
+
+    def render(self, request):
+        d = defer.maybeDeferred(lambda : self.performAction(request))
+        def redirect(url):
+            request.redirect(url)
+            request.write("see <a href='%s'>%s</a>" % (url,url))
+            try:
+                request.finish()
+            except RuntimeError:
+                # this occurs when the client has already disconnected; ignore
+                # it (see #2027)
+                log.msg("http client disconnected before results were sent")
+        d.addCallback(redirect)
+
+        def fail(f):
+            request.processingFailed(f)
+            return None # processingFailed will log this for us
+        d.addErrback(fail)
+        return server.NOT_DONE_YET
+
 class HtmlResource(resource.Resource, ContextMixin):
     # this is a cheap sort of template thingy
     contentType = "text/html; charset=utf-8"
-    title = "Buildbot"
+    pageTitle = "Buildbot"
     addSlash = False # adapted from Nevow
 
     def getChild(self, path, request):
@@ -271,24 +312,38 @@ class HtmlResource(resource.Resource, ContextMixin):
 
         ctx = self.getContext(request)
 
-        data = self.content(request, ctx)
-        if isinstance(data, unicode):
-            data = data.encode("utf-8")
-        request.setHeader("content-type", self.contentType)
-        if request.method == "HEAD":
-            request.setHeader("content-length", len(data))
-            return ''
-        return data
-
+        d = defer.maybeDeferred(lambda : self.content(request, ctx))
+        def handle(data):
+            if isinstance(data, unicode):
+                data = data.encode("utf-8")
+            request.setHeader("content-type", self.contentType)
+            if request.method == "HEAD":
+                request.setHeader("content-length", len(data))
+                return ''
+            return data
+        d.addCallback(handle)
+        def ok(data):
+            request.write(data)
+            try:
+                request.finish()
+            except RuntimeError:
+                # this occurs when the client has already disconnected; ignore
+                # it (see #2027)
+                log.msg("http client disconnected before results were sent")
+        def fail(f):
+            request.processingFailed(f)
+            return None # processingFailed will log this for us
+        d.addCallbacks(ok, fail)
+        return server.NOT_DONE_YET
 
 class StaticHTML(HtmlResource):
-    def __init__(self, body, title):
+    def __init__(self, body, pageTitle):
         HtmlResource.__init__(self)
         self.bodyHTML = body
-        self.title = title
+        self.pageTitle = pageTitle
     def content(self, request, cxt):
         cxt['content'] = self.bodyHTML
-        cxt['title'] = self.title
+        cxt['pageTitle'] = self.pageTitle
         template = request.site.buildbot_service.templates.get_template("empty.html")
         return template.render(**cxt)
 
@@ -301,7 +356,7 @@ if hasattr(static, 'DirectoryLister'):
         """This variant of the static.DirectoryLister uses a template
         for rendering."""
 
-        title = 'BuildBot'
+        pageTitle = 'BuildBot'
 
         def render(self, request):
             cxt = self.getContext(request)
@@ -376,13 +431,7 @@ class BuildLineMixin:
         builder_name = build.getBuilder().getName()
         results = build.getResults()
         text = build.getText()
-        try:
-            rev = build.getProperty("got_revision")
-            if rev is None:
-                rev = "??"
-        except KeyError:
-            rev = "??"
-        rev = str(rev)
+        rev = str(build.getProperty("got_revision", "??"))
         css_class = css_classes.get(results, "")
         repo = build.getSourceStamp().repository
 
