@@ -1,18 +1,126 @@
+# This file is part of Buildbot.  Buildbot is free software: you can
+# redistribute it and/or modify it under the terms of the GNU General Public
+# License as published by the Free Software Foundation, version 2.
+#
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
+#
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51
+# Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+#
+# Copyright Buildbot Team Members
+
 
 from twisted.web import html
-from twisted.web.util import Redirect, DeferredResource
 from twisted.internet import defer, reactor
+from twisted.web.util import Redirect, DeferredResource
 
 import urllib, time
 from twisted.python import log
 from buildbot.status.web.base import HtmlResource, \
      css_classes, path_to_build, path_to_builder, path_to_slave, \
-     getAndCheckProperties, path_to_authfail
+     getAndCheckProperties, ActionResource, path_to_authfail
 
 from buildbot.status.web.step import StepsResource
+from buildbot.status.web.tests import TestsResource
 from buildbot import util, interfaces
 
+class ForceBuildActionResource(ActionResource):
 
+    def __init__(self, build_status, builder):
+        self.build_status = build_status
+        self.builder = builder
+        self.action = "forceBuild"
+
+    @defer.deferredGenerator
+    def performAction(self, req):
+        url = None
+        d = self.getAuthz(req).actionAllowed(self.action, req, self.builder)
+        wfd = defer.waitForDeferred(d)
+        yield wfd
+        res = wfd.getResult()
+
+        if not res:
+            url = path_to_authfail(req)
+        else:
+            # get a control object
+            c = interfaces.IControl(self.getBuildmaster(req))
+            bc = c.getBuilder(self.builder.getName())
+
+            b = self.build_status
+            builder_name = self.builder.getName()
+            log.msg("web rebuild of build %s:%s" % (builder_name, b.getNumber()))
+            name = req.args.get("username", ["<unknown>"])[0]
+            comments = req.args.get("comments", ["<no reason specified>"])[0]
+            reason = ("The web-page 'rebuild' button was pressed by "
+                      "'%s': %s\n" % (name, comments))
+            extraProperties = getAndCheckProperties(req)
+            if not bc or not b.isFinished() or extraProperties is None:
+                log.msg("could not rebuild: bc=%s, isFinished=%s"
+                        % (bc, b.isFinished()))
+                # TODO: indicate an error
+            else:
+                d = bc.rebuildBuild(b, reason, extraProperties)
+                wfd = defer.waitForDeferred(d)
+                yield wfd
+                tup = wfd.getResult()
+                # check that (bsid, brids) were properly stored
+                if not isinstance(tup, (int, dict)):
+                    log.err("while rebuilding a build")
+            # we're at
+            # http://localhost:8080/builders/NAME/builds/5/rebuild?[args]
+            # Where should we send them?
+            #
+            # Ideally it would be to the per-build page that they just started,
+            # but we don't know the build number for it yet (besides, it might
+            # have to wait for a current build to finish). The next-most
+            # preferred place is somewhere that the user can see tangible
+            # evidence of their build starting (or to see the reason that it
+            # didn't start). This should be the Builder page.
+
+            url = path_to_builder(req, self.builder)
+        yield url
+
+
+class StopBuildActionResource(ActionResource):
+
+    def __init__(self, build_status):
+        self.build_status = build_status
+        self.action = "stopBuild"
+
+    @defer.deferredGenerator
+    def performAction(self, req):
+        d = self.getAuthz(req).actionAllowed(self.action, req,
+                                             self.build_status)
+        wfd = defer.waitForDeferred(d)
+        yield wfd
+        res = wfd.getResult()
+
+        if not res:
+            yield path_to_authfail(req)
+            return
+
+        b = self.build_status
+        log.msg("web stopBuild of build %s:%s" % \
+                    (b.getBuilder().getName(), b.getNumber()))
+        name = req.args.get("username", ["<unknown>"])[0]
+        comments = req.args.get("comments", ["<no reason specified>"])[0]
+        # html-quote both the username and comments, just to be safe
+        reason = ("The web-page 'stop build' button was pressed by "
+                  "'%s': %s\n" % (html.escape(name), html.escape(comments)))
+
+        c = interfaces.IControl(self.getBuildmaster(req))
+        bldrc = c.getBuilder(self.build_status.getBuilder().getName())
+        if bldrc:
+            bldc = bldrc.getBuild(self.build_status.getNumber())
+            if bldc:
+                bldc.stopBuild(reason)
+
+        yield path_to_builder(req, self.build_status.getBuilder())
+        return
 
 # /builders/$builder/builds/$buildnum
 class StatusResourceBuild(HtmlResource):
@@ -22,7 +130,7 @@ class StatusResourceBuild(HtmlResource):
         HtmlResource.__init__(self)
         self.build_status = build_status
 
-    def getTitle(self, request):
+    def getPageTitle(self, request):
         return ("Buildbot: %s Build #%d" %
                 (self.build_status.getBuilder().getName(),
                  self.build_status.getNumber()))
@@ -30,33 +138,38 @@ class StatusResourceBuild(HtmlResource):
     def content(self, req, cxt):
         b = self.build_status
         status = self.getStatus(req)
+        req.setHeader('Cache-Control', 'no-cache')
 
         cxt['b'] = b
         cxt['path_to_builder'] = path_to_builder(req, b.getBuilder())
-        
+
         if not b.isFinished():
+            step = b.getCurrentStep()
+            if not step:
+                cxt['current_step'] = "[waiting for Lock]"
+            else:
+                if step.isWaitingForLocks():
+                    cxt['current_step'] = "%s [waiting for Lock]" % step.getName()
+                else:
+                    cxt['current_step'] = step.getName()
             when = b.getETA()
             if when is not None:
                 cxt['when'] = util.formatInterval(when)
                 cxt['when_time'] = time.strftime("%H:%M:%S",
                                                 time.localtime(time.time() + when))
-                
-        else: 
+
+        else:
             cxt['result_css'] = css_classes[b.getResults()]
             if b.getTestResults():
                 cxt['tests_link'] = req.childLink("tests")
-                
+
         ss = cxt['ss'] = b.getSourceStamp()
-        
+
         if ss.branch is None and ss.revision is None and ss.patch is None and not ss.changes:
             cxt['most_recent_rev_build'] = True
-            
-        
-        got_revision = None
-        try:
-            got_revision = b.getProperty("got_revision")
-        except KeyError:
-            pass
+
+
+        got_revision = b.getProperty("got_revision")
         if got_revision:
             cxt['got_revision'] = str(got_revision)
 
@@ -76,13 +189,18 @@ class StatusResourceBuild(HtmlResource):
                 (start, end) = s.getTimes()
                 step['time_to_run'] = util.formatInterval(end - start)
             elif s.isStarted():
-                step['css_class'] = "running"
-                step['time_to_run'] = "running"
+                if s.isWaitingForLocks():
+                    step['css_class'] = "waiting"
+                    step['time_to_run'] = "waiting for locks"
+                else:
+                    step['css_class'] = "running"
+                    step['time_to_run'] = "running"
             else:
                 step['css_class'] = "not_started"
                 step['time_to_run'] = ""
 
-            step['link'] = req.childLink("steps/%s" % urllib.quote(s.getName()))
+            step['link'] = req.childLink("steps/%s" % 
+                                    urllib.quote(s.getName(), safe=''))
             step['text'] = " ".join(s.getText())
             step['urls'] = map(lambda x:dict(url=x[1],logname=x[0]), s.getURLs().items())
 
@@ -90,13 +208,13 @@ class StatusResourceBuild(HtmlResource):
             for l in s.getLogs():
                 logname = l.getName()
                 step['logs'].append({ 'link': req.childLink("steps/%s/logs/%s" %
-                                           (urllib.quote(s.getName()),
-                                            urllib.quote(logname))), 
+                                           (urllib.quote(s.getName(), safe=''),
+                                            urllib.quote(logname, safe=''))), 
                                       'name': logname })
 
         ps = cxt['properties'] = []
         for name, value, source in b.getProperties().asList():
-            value = str(value)
+            value = unicode(value)
             p = { 'name': name, 'value': value, 'source': source}            
             if len(value) > 500:
                 p['short_value'] = value[:500]
@@ -126,8 +244,7 @@ class StatusResourceBuild(HtmlResource):
     def stop(self, req, auth_ok=False):
         # check if this is allowed
         if not auth_ok:
-            if not self.getAuthz(req).actionAllowed('stopBuild', req, self.build_status):
-                return Redirect(path_to_authfail(req))
+            return StopBuildActionResource(self.build_status)
 
         b = self.build_status
         log.msg("web stopBuild of build %s:%s" % \
@@ -153,43 +270,8 @@ class StatusResourceBuild(HtmlResource):
         return DeferredResource(d)
 
     def rebuild(self, req):
-        # check auth
-        if not self.getAuthz(req).actionAllowed('forceBuild', req, self.build_status.getBuilder()):
-            return Redirect(path_to_authfail(req))
-
-        # get a control object
-        c = interfaces.IControl(self.getBuildmaster(req))
-        bc = c.getBuilder(self.build_status.getBuilder().getName())
-
-        b = self.build_status
-        builder_name = b.getBuilder().getName()
-        log.msg("web rebuild of build %s:%s" % (builder_name, b.getNumber()))
-        name = req.args.get("username", ["<unknown>"])[0]
-        comments = req.args.get("comments", ["<no reason specified>"])[0]
-        reason = ("The web-page 'rebuild' button was pressed by "
-                  "'%s': %s\n" % (name, comments))
-        extraProperties = getAndCheckProperties(req)
-        if not bc or not b.isFinished() or extraProperties is None:
-            log.msg("could not rebuild: bc=%s, isFinished=%s"
-                    % (bc, b.isFinished()))
-            # TODO: indicate an error
-        else:
-            bc.rebuildBuild(b, reason, extraProperties)
-        # we're at
-        # http://localhost:8080/builders/NAME/builds/5/rebuild?[args]
-        # Where should we send them?
-        #
-        # Ideally it would be to the per-build page that they just started,
-        # but we don't know the build number for it yet (besides, it might
-        # have to wait for a current build to finish). The next-most
-        # preferred place is somewhere that the user can see tangible
-        # evidence of their build starting (or to see the reason that it
-        # didn't start). This should be the Builder page.
-
-        r = Redirect(path_to_builder(req, self.build_status.getBuilder()))
-        d = defer.Deferred()
-        reactor.callLater(1, d.callback, r)
-        return DeferredResource(d)
+        return ForceBuildActionResource(self.build_status,
+                                        self.build_status.getBuilder())
 
     def getChild(self, path, req):
         if path == "stop":
@@ -198,6 +280,8 @@ class StatusResourceBuild(HtmlResource):
             return self.rebuild(req)
         if path == "steps":
             return StepsResource(self.build_status)
+        if path == "tests":
+            return TestsResource(self.build_status)
 
         return HtmlResource.getChild(self, path, req)
 
