@@ -17,7 +17,7 @@
 # code to deliver build status through twisted.words (instant messaging
 # protocols: irc, etc)
 
-import re, shlex
+import re, shlex, random
 from string import join, capitalize, lower
 
 from zope.interface import Interface, implements
@@ -31,8 +31,9 @@ from buildbot import version
 from buildbot.interfaces import IStatusReceiver
 from buildbot.sourcestamp import SourceStamp
 from buildbot.status import base
-from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, EXCEPTION
+from buildbot.status.results import SUCCESS, WARNINGS, FAILURE, EXCEPTION, RETRY
 from buildbot.scripts.runner import ForceOptions
+from buildbot.process.properties import Properties
 
 # twisted.internet.ssl requires PyOpenSSL, so be resilient if it's missing
 try:
@@ -49,8 +50,9 @@ class IrcBuildRequest:
     hasStarted = False
     timer = None
 
-    def __init__(self, parent):
+    def __init__(self, parent, useRevisions=False):
         self.parent = parent
+        self.useRevisions = useRevisions
         self.timer = reactor.callLater(5, self.soon)
 
     def soon(self):
@@ -65,7 +67,10 @@ class IrcBuildRequest:
             self.timer.cancel()
             del self.timer
         eta = s.getETA()
-        response = "build #%d forced" % s.getNumber()
+        if self.useRevisions:
+            response = "build containing revision(s) [%s] forced" % s.getRevisions()
+        else:
+            response = "build #%d forced" % s.getNumber()
         if eta is not None:
             response = "build forced [ETA %s]" % self.parent.convertTime(eta)
         self.parent.send(response)
@@ -89,8 +94,11 @@ class Contact(base.StatusReceiver):
     def __init__(self, channel):
         #StatusReceiver.__init__(self) doesn't exist
         self.channel = channel
+        self.master = channel.master
         self.notify_events = {}
         self.subscribed = 0
+        self.muted = False
+        self.useRevisions = channel.useRevisions
         self.reported_builds = [] # tuples (when, buildername, buildnum)
         self.add_notification_events(channel.notify_events)
 
@@ -296,8 +304,12 @@ class Contact(base.StatusReceiver):
             assert not build.isFinished()
             d = build.waitUntilFinished()
             d.addCallback(self.watchedBuildFinished)
-            r = "watching build %s #%d until it finishes" \
-                % (which, build.getNumber())
+            if self.useRevisions:
+                r = "watching build %s containing revision(s) [%s] until it finishes" \
+                    % (which, build.getRevisions())
+            else:
+                r = "watching build %s #%d until it finishes" \
+                    % (which, build.getNumber())
             eta = build.getETA()
             if eta is not None:
                 r += " [%s]" % self.convertTime(eta)
@@ -317,7 +329,7 @@ class Contact(base.StatusReceiver):
 
     def requestSubmitted(self, brstatus):
         log.msg('[Contact] BuildRequest %d for %s submitted' %
-            (brstatus.brid, brstatus.getSourceStamp()))
+            (brstatus.brid, brstatus.getBuilderName()))
 
     def builderRemoved(self, builderName):
         log.msg('[Contact] Builder %s removed' % (builderName))
@@ -337,11 +349,15 @@ class Contact(base.StatusReceiver):
             log.msg('Not notifying for a build when started-notification disabled')
             return
 
-        r = "build #%d of %s started, including [%s]" % \
-           (build.getNumber(),
-            builder.getName(),
-            ", ".join([str(c.revision) for c in build.getChanges()])
-            )
+        if self.useRevisions:
+            r = "build containing revision(s) [%s] on %s started" % \
+                (build.getRevisions(), builder.getName())
+        else:
+            r = "build #%d of %s started, including [%s]" % \
+                (build.getNumber(),
+                 builder.getName(),
+                 ", ".join([str(c.revision) for c in build.getChanges()])
+                 )
 
         self.send(r)
 
@@ -350,6 +366,7 @@ class Contact(base.StatusReceiver):
         WARNINGS: "Warnings",
         FAILURE: "Failure",
         EXCEPTION: "Exception",
+        RETRY: "Retry",
         }
 
     def buildFinished(self, builderName, build, results):
@@ -367,12 +384,19 @@ class Contact(base.StatusReceiver):
 
         builder_name = builder.getName()
         buildnum = build.getNumber()
+        buildrevs = build.getRevisions()
 
         if self.reportBuild(builder_name, buildnum):
-            r = "build #%d of %s is complete: %s" % \
-                (buildnum,
-                 builder_name,
-                 self.results_descriptions.get(build.getResults(), "??"))
+            if self.useRevisions:
+                r = "build containing revision(s) [%s] on %s is complete: %s" % \
+                    (buildrevs,
+                     builder_name,
+                     self.results_descriptions.get(build.getResults(), "??"))
+            else:
+                r = "build #%d of %s is complete: %s" % \
+                    (buildnum,
+                     builder_name,
+                     self.results_descriptions.get(build.getResults(), "??"))
             r += " [%s]" % " ".join(build.getText())
             buildurl = self.channel.status.getURLForThing(build)
             if buildurl:
@@ -418,12 +442,19 @@ class Contact(base.StatusReceiver):
 
         builder_name = b.getBuilder().getName()
         buildnum = b.getNumber()
+        buildrevs = b.getRevisions()
 
         if self.reportBuild(builder_name, buildnum):
-            r = "Hey! build %s #%d is complete: %s" % \
-                (builder_name, 
-                 buildnum,
-                 self.results_descriptions.get(b.getResults(), "??"))
+            if self.useRevisions:
+                r = "Hey! build %s containing revision(s) [%s] is complete: %s" % \
+                    (builder_name, 
+                     buildrevs,
+                     self.results_descriptions.get(b.getResults(), "??"))
+            else:
+                r = "Hey! build %s #%d is complete: %s" % \
+                    (builder_name, 
+                     buildnum,
+                     self.results_descriptions.get(b.getResults(), "??"))
             r += " [%s]" % " ".join(b.getText())
             self.send(r)
             buildurl = self.channel.status.getURLForThing(b)
@@ -431,12 +462,13 @@ class Contact(base.StatusReceiver):
                 self.send("Build details are at %s" % buildurl)
 
     def command_FORCE(self, args, who):
+        errReply = "try 'force build [--branch=BRANCH] [--revision=REVISION] [--props=PROP1=VAL1,PROP2=VAL2...]  <WHICH> <REASON>'"
         args = shlex.split(args)
         if not args:
-            raise UsageError("try 'force build WHICH <REASON>'")
+            raise UsageError(errReply)
         what = args.pop(0)
         if what != "build":
-            raise UsageError("try 'force build WHICH <REASON>'")
+            raise UsageError(errReply)
         opts = ForceOptions()
         opts.parseOptions(args)
 
@@ -444,38 +476,60 @@ class Contact(base.StatusReceiver):
         branch = opts['branch']
         revision = opts['revision']
         reason = opts['reason']
+        props = opts['props']
 
         if which is None:
-            raise UsageError("you must provide a Builder, "
-                             "try 'force build WHICH <REASON>'")
+            raise UsageError("you must provide a Builder, " + errReply)
 
-        # keep weird stuff out of the branch and revision strings. 
-        # TODO:  centralize this somewhere.
-        if branch and not re.match(r'^[\w\.\-\/]*$', branch):
+        # keep weird stuff out of the branch, revision, and properties args.
+        branch_validate = self.master.config.validation['branch']
+        revision_validate = self.master.config.validation['revision']
+        pname_validate = self.master.config.validation['property_name']
+        pval_validate = self.master.config.validation['property_value']
+        if branch and not branch_validate.match(branch):
             log.msg("bad branch '%s'" % branch)
             self.send("sorry, bad branch '%s'" % branch)
             return
-        if revision and not re.match(r'^[\w\.\-\/]*$', revision):
+        if revision and not revision_validate.match(revision):
             log.msg("bad revision '%s'" % revision)
             self.send("sorry, bad revision '%s'" % revision)
             return
 
+        properties = None
+        if props:
+            # split props into name:value dict
+            pdict = {}
+            propertylist = props.split(",")
+            for i in range(0,len(propertylist)):
+                splitproperty = propertylist[i].split("=", 1)
+                pdict[splitproperty[0]] = splitproperty[1]
+
+            # set properties
+            properties = Properties()
+            for prop in pdict:
+                pname = prop
+                pvalue = pdict[prop]
+                if not pname_validate.match(pname) \
+                        or not pval_validate.match(pvalue):
+                    log.msg("bad property name='%s', value='%s'" % (pname, pvalue))
+                    self.send("sorry, bad property name='%s', value='%s'" %
+                              (pname, pvalue))
+                    return
+                properties.setProperty(pname, pvalue, "Force Build IRC")
+
         bc = self.getControl(which)
 
-        # TODO: maybe give certain users the ability to request builds of
-        # certain branches
         reason = "forced: by %s: %s" % (self.describeUser(who), reason)
         ss = SourceStamp(branch=branch, revision=revision)
-        try:
-            brs = bc.submitBuildRequest(ss, reason, now=True)
-        except interfaces.NoSlaveError:
-            self.send("sorry, I can't force a build: all slaves are offline")
-            return
-        ireq = IrcBuildRequest(self)
-        brs.subscribe(ireq.started)
+        d = bc.submitBuildRequest(ss, reason, props=properties.asDict())
+        def subscribe(buildreq):
+            ireq = IrcBuildRequest(self, self.useRevisions)
+            buildreq.subscribe(ireq.started)
+        d.addCallback(subscribe)
+        d.addErrback(log.err, "while forcing a build")
 
 
-    command_FORCE.usage = "force build <which> <reason> - Force a build"
+    command_FORCE.usage = "force build [--branch=branch] [--revision=revision] [--props=prop1=val1,prop2=val2...] <which> <reason> - Force a build"
 
     def command_STOP(self, args, who):
         args = shlex.split(args)
@@ -496,6 +550,7 @@ class Contact(base.StatusReceiver):
             return
         for build in builds:
             num = build.getNumber()
+            revs = build.getRevisions()
 
             # obtain the BuildControl object
             buildcontrol = buildercontrol.getBuild(num)
@@ -503,7 +558,11 @@ class Contact(base.StatusReceiver):
             # make it stop
             buildcontrol.stopBuild(r)
 
-            self.send("build %d interrupted" % num)
+            if self.useRevisions:
+                response = "build containing revision(s) [%s] interrupted" % revs
+            else:
+                response = "build %d interrupted" % num
+            self.send(response)
 
     command_STOP.usage = "stop build <which> <reason> - Stop a running build"
 
@@ -559,6 +618,21 @@ class Contact(base.StatusReceiver):
         self.emit_last(which)
     command_LAST.usage = "last <which> - list last build status for builder <which>"
 
+    def command_MUTE(self, args, who):
+        # The order of these is important! ;)
+        self.send("Shutting up for now.")
+        self.muted = True
+    command_MUTE.usage = "mute - suppress all messages until a corresponding 'unmute' is issued"
+
+    def command_UNMUTE(self, args, who):
+        if self.muted:
+            # The order of these is important! ;)
+            self.muted = False
+            self.send("I'm baaaaaaaaaaack!")
+        else:
+            self.send("You hadn't told me to be quiet, but it's the thought that counts, right?")
+    command_UNMUTE.usage = "unmute - disable a previous 'mute'"
+
     def build_commands(self):
         commands = []
         for k in dir(self):
@@ -645,10 +719,12 @@ class IRCContact(Contact):
     # userJoined(self, user, channel)
 
     def send(self, message):
-        self.channel.msgOrNotice(self.dest, message.encode("ascii", "replace"))
+        if not self.muted:
+            self.channel.msgOrNotice(self.dest, message.encode("ascii", "replace"))
 
     def act(self, action):
-        self.channel.me(self.dest, action.encode("ascii", "replace"))
+        if not self.muted:
+            self.channel.me(self.dest, action.encode("ascii", "replace"))
 
     def handleMessage(self, message, who):
         # a message has arrived from 'who'. For broadcast contacts (i.e. when
@@ -706,28 +782,40 @@ class IrcStatusBot(irc.IRCClient):
     implements(IChannel)
     contactClass = IRCContact
 
-    def __init__(self, nickname, password, channels, status, categories, notify_events, noticeOnChannel = False, showBlameList = False):
+    def __init__(self, nickname, password, channels, status, categories,
+                 notify_events, noticeOnChannel=False, useRevisions=False,
+                 showBlameList=False):
         """
         @type  nickname: string
         @param nickname: the nickname by which this bot should be known
         @type  password: string
         @param password: the password to use for identifying with Nickserv
-        @type  channels: list of strings
+        @type  channels: list of dictionaries
         @param channels: the bot will maintain a presence in these channels
         @type  status: L{buildbot.status.builder.Status}
         @param status: the build master's Status object, through which the
                        bot retrieves all status information
+        @type  noticeOnChannel: boolean
+        @param noticeOnChannel: Defaults to False. If True, error messages
+                                for bot commands will be sent to the channel
+                                as notices. Otherwise they are sent as a msg.
+        @type  useRevisions: boolean
+        @param useRevisions: if True, messages from the bot will use the
+                             revisions from the Changes in the build and not
+                             the build number.
         """
         self.nickname = nickname
         self.channels = channels
         self.password = password
         self.status = status
+        self.master = status.master
         self.categories = categories
         self.notify_events = notify_events
         self.counter = 0
         self.hasQuit = 0
         self.contacts = {}
         self.noticeOnChannel = noticeOnChannel
+        self.useRevisions = useRevisions
         self.showBlameList = showBlameList
 
     def msgOrNotice(self, dest, message):
@@ -784,13 +872,17 @@ class IrcStatusBot(irc.IRCClient):
         if "buildbot" in data:
             contact.handleAction(data, user)
 
-
-
     def signedOn(self):
         if self.password:
             self.msg("Nickserv", "IDENTIFY " + self.password)
         for c in self.channels:
-            self.join(c)
+            if isinstance(c, dict):
+                channel = c.get('channel', None)
+                password = c.get('password', None)
+            else:
+                channel = c
+                password = None
+            self.join(channel=channel, key=password)
 
     def joined(self, channel):
         self.log("I have joined %s" % (channel,))
@@ -814,10 +906,18 @@ class IrcStatusBot(irc.IRCClient):
     # self.quit(message='')
 
 class ThrottledClientFactory(protocol.ClientFactory):
-    lostDelay = 2
-    failedDelay = 60
+    lostDelay = random.randint(1, 5)
+    failedDelay = random.randint(45, 60)
+
+    def __init__(self, lostDelay=None, failedDelay=None):
+        if lostDelay is not None:
+            self.lostDelay = lostDelay
+        if failedDelay is not None:
+            self.failedDelay = failedDelay
+
     def clientConnectionLost(self, connector, reason):
         reactor.callLater(self.lostDelay, connector.connect)
+
     def clientConnectionFailed(self, connector, reason):
         reactor.callLater(self.failedDelay, connector.connect)
 
@@ -829,8 +929,11 @@ class IrcStatusFactory(ThrottledClientFactory):
     shuttingDown = False
     p = None
 
-    def __init__(self, nickname, password, channels, categories, notify_events, noticeOnChannel = False, showBlameList = False):
-        #ThrottledClientFactory.__init__(self) # doesn't exist
+    def __init__(self, nickname, password, channels, categories, notify_events,
+                 noticeOnChannel=False, useRevisions=False, showBlameList=False,
+                 lostDelay=None, failedDelay=None):
+        ThrottledClientFactory.__init__(self, lostDelay=lostDelay,
+                                        failedDelay=failedDelay)
         self.status = None
         self.nickname = nickname
         self.password = password
@@ -838,6 +941,7 @@ class IrcStatusFactory(ThrottledClientFactory):
         self.categories = categories
         self.notify_events = notify_events
         self.noticeOnChannel = noticeOnChannel
+        self.useRevisions = useRevisions
         self.showBlameList = showBlameList
 
     def __getstate__(self):
@@ -855,6 +959,7 @@ class IrcStatusFactory(ThrottledClientFactory):
                           self.channels, self.status,
                           self.categories, self.notify_events,
                           noticeOnChannel = self.noticeOnChannel,
+                          useRevisions = self.useRevisions,
                           showBlameList = self.showBlameList)
         p.factory = self
         p.status = self.status
@@ -888,12 +993,14 @@ class IRC(base.StatusReceiverMultiService):
 
     compare_attrs = ["host", "port", "nick", "password",
                      "channels", "allowForce", "useSSL",
-                     "categories"]
+                     "useRevisions", "categories",
+                     "lostDelay", "failedDelay"]
 
-    def __init__(self, host, nick, channels, port=6667, allowForce=True,
+    def __init__(self, host, nick, channels, port=6667, allowForce=False,
                  categories=None, password=None, notify_events={},
                  noticeOnChannel = False, showBlameList = True,
-                 useSSL=False):
+                 useRevisions=False, useSSL=False,
+                 lostDelay=None, failedDelay=None):
         base.StatusReceiverMultiService.__init__(self)
 
         assert allowForce in (True, False) # TODO: implement others
@@ -905,13 +1012,16 @@ class IRC(base.StatusReceiverMultiService):
         self.channels = channels
         self.password = password
         self.allowForce = allowForce
+        self.useRevisions = useRevisions
         self.categories = categories
         self.notify_events = notify_events
         log.msg('Notify events %s' % notify_events)
         self.f = IrcStatusFactory(self.nick, self.password,
                                   self.channels, self.categories, self.notify_events,
                                   noticeOnChannel = noticeOnChannel,
-                                  showBlameList = showBlameList)
+                                  useRevisions = useRevisions,
+                                  showBlameList = showBlameList,
+                                  lostDelay = lostDelay, failedDelay = failedDelay)
 
         # don't set up an actual ClientContextFactory if we're running tests.
         if self.in_test_harness:
