@@ -16,10 +16,9 @@
 
 from twisted.python import log
 
-from buildbot.status import builder
-from buildbot.status.builder import SUCCESS, FAILURE, WARNINGS, SKIPPED
+from buildbot.status import testresult
+from buildbot.status.results import SUCCESS, FAILURE, WARNINGS, SKIPPED
 from buildbot.process.buildstep import LogLineObserver, OutputProgressObserver
-from buildbot.process.buildstep import RemoteShellCommand
 from buildbot.steps.shell import ShellCommand
 
 try:
@@ -193,6 +192,7 @@ class Trial(ShellCommand):
     logfiles = {"test.log": "_trial_temp/test.log"}
     # we use test.log to track Progress at the end of __init__()
 
+    renderables = ['tests']
     flunkOnFailure = True
     python = None
     trial = "trial"
@@ -383,22 +383,14 @@ class Trial(ShellCommand):
             if e is None:
                 cmd.args['env'] = {'PYTHONPATH': self.testpath}
             else:
-                # TODO: somehow, each build causes another copy of
-                # self.testpath to get prepended
-                if e.get('PYTHONPATH', "") == "":
-                    e['PYTHONPATH'] = self.testpath
-                else:
-                    e['PYTHONPATH'] = self.testpath + ":" + e['PYTHONPATH']
-        try:
-            p = cmd.args['env']['PYTHONPATH']
-            if type(p) is not str:
-                log.msg("hey, not a string:", p)
-                assert False
-        except (KeyError, TypeError):
-            # KeyError if args doesn't have ['env']
-            # KeyError if args['env'] doesn't have ['PYTHONPATH']
-            # TypeError if args is None
-            pass
+                #this bit produces a list, which can be used
+                #by buildslave.runprocess.RunProcess
+                ppath = e.get('PYTHONPATH', self.testpath)
+                if isinstance(ppath, str):
+                    ppath = [ppath]
+                if self.testpath not in ppath:
+                    ppath.insert(0, self.testpath)
+                e['PYTHONPATH'] = ppath
 
     def start(self):
         # now that self.build.allFiles() is nailed down, finish building the
@@ -411,41 +403,10 @@ class Trial(ShellCommand):
             self.command.extend(self.tests)
         log.msg("Trial.start: command is", self.command)
 
-        # if our slave is too old to understand logfiles=, fetch them
-        # manually. This is a fallback for the Twisted buildbot and some old
-        # buildslaves.
-        self._needToPullTestDotLog = False
-        if self.slaveVersionIsOlderThan("shell", "2.1"):
-            log.msg("Trial: buildslave %s is too old to accept logfiles=" %
-                    self.getSlaveName())
-            log.msg(" falling back to 'cat _trial_temp/test.log' instead")
-            self.logfiles = {}
-            self._needToPullTestDotLog = True
-
         ShellCommand.start(self)
 
 
     def commandComplete(self, cmd):
-        if not self._needToPullTestDotLog:
-            return self._gotTestDotLog(cmd)
-
-        # if the buildslave was too old, pull test.log now
-        catcmd = ["cat", "_trial_temp/test.log"]
-        c2 = RemoteShellCommand(command=catcmd, workdir=self.workdir)
-        loog = self.addLog("test.log")
-        c2.useLog(loog, True, logfileName="stdio")
-        self.cmd = c2 # to allow interrupts
-        d = c2.run(self, self.remote)
-        d.addCallback(lambda res: self._gotTestDotLog(cmd))
-        return d
-
-    def rtext(self, fmt='%s'):
-        if self.reactor:
-            rtext = fmt % self.reactor
-            return rtext.replace("reactor", "")
-        return ""
-
-    def _gotTestDotLog(self, cmd):
         # figure out all status, then let the various hook functions return
         # different pieces of it
 
@@ -525,10 +486,17 @@ class Trial(ShellCommand):
         self.text = text
         self.text2 = [text2]
 
+
+    def rtext(self, fmt='%s'):
+        if self.reactor:
+            rtext = fmt % self.reactor
+            return rtext.replace("reactor", "")
+        return ""
+
     def addTestResult(self, testname, results, text, tlog):
         if self.reactor is not None:
             testname = (self.reactor,) + testname
-        tr = builder.TestResult(testname, results, text, logs={'log': tlog})
+        tr = testresult.TestResult(testname, results, text, logs={'log': tlog})
         #self.step_status.build.addTestResult(tr)
         self.build.build_status.addTestResult(tr)
 
