@@ -63,6 +63,9 @@ class _FileWriter(pb.Referenceable):
         else:
             self.fp.write(data)
 
+    def remote_utime(self, accessed_modified):
+        os.utime(self.destfile,accessed_modified)
+
     def remote_close(self):
         """
         Called by remote slave to state that no more data will be transfered
@@ -185,17 +188,27 @@ class _TransferBuildStep(BuildStep):
     """
     DEFAULT_WORKDIR = "build"           # is this redundant?
 
+    renderables = [ 'workdir' ]
+
+    haltOnFailure = True
+    flunkOnFailure = True
+
     def setDefaultWorkdir(self, workdir):
         if self.workdir is None:
             self.workdir = workdir
 
     def _getWorkdir(self):
-        properties = self.build.getProperties()
         if self.workdir is None:
             workdir = self.DEFAULT_WORKDIR
         else:
             workdir = self.workdir
-        return properties.render(workdir)
+        return workdir
+
+    def interrupt(self, reason):
+        self.addCompleteLog('interrupt', str(reason))
+        if self.cmd:
+            d = self.cmd.interrupt(reason)
+            return d
 
     def finished(self, result):
         # Subclasses may choose to skip a transfer. In those cases, self.cmd
@@ -212,27 +225,13 @@ class _TransferBuildStep(BuildStep):
 
 
 class FileUpload(_TransferBuildStep):
-    """
-    Build step to transfer a file from the slave to the master.
-
-    arguments:
-
-    - ['slavesrc']   filename of source file at slave, relative to workdir
-    - ['masterdest'] filename of destination file at master
-    - ['workdir']    string with slave working directory relative to builder
-                     base dir, default 'build'
-    - ['maxsize']    maximum size of the file, default None (=unlimited)
-    - ['blocksize']  maximum size of each block being transfered
-    - ['mode']       file access mode for the resulting master-side file.
-                     The default (=None) is to leave it up to the umask of
-                     the buildmaster process.
-
-    """
 
     name = 'upload'
 
+    renderables = [ 'slavesrc', 'masterdest' ]
+
     def __init__(self, slavesrc, masterdest,
-                 workdir=None, maxsize=None, blocksize=16*1024, mode=None,
+                 workdir=None, maxsize=None, blocksize=16*1024, mode=None, keepstamp=False,
                  **buildstep_kwargs):
         BuildStep.__init__(self, **buildstep_kwargs)
         self.addFactoryArguments(slavesrc=slavesrc,
@@ -241,6 +240,7 @@ class FileUpload(_TransferBuildStep):
                                  maxsize=maxsize,
                                  blocksize=blocksize,
                                  mode=mode,
+                                 keepstamp=keepstamp,
                                  )
 
         self.slavesrc = slavesrc
@@ -250,17 +250,17 @@ class FileUpload(_TransferBuildStep):
         self.blocksize = blocksize
         assert isinstance(mode, (int, type(None)))
         self.mode = mode
+        self.keepstamp = keepstamp
 
     def start(self):
         version = self.slaveVersion("uploadFile")
-        properties = self.build.getProperties()
 
         if not version:
             m = "slave is too old, does not know about uploadFile"
             raise BuildSlaveTooOldError(m)
 
-        source = properties.render(self.slavesrc)
-        masterdest = properties.render(self.masterdest)
+        source = self.slavesrc
+        masterdest = self.masterdest
         # we rely upon the fact that the buildmaster runs chdir'ed into its
         # basedir to make sure that relative paths in masterdest are expanded
         # properly. TODO: maybe pass the master's basedir all the way down
@@ -274,6 +274,11 @@ class FileUpload(_TransferBuildStep):
         # we use maxsize to limit the amount of data on both sides
         fileWriter = _FileWriter(masterdest, self.maxsize, self.mode)
 
+        if self.keepstamp and self.slaveVersionIsOlderThan("uploadFile","2.13"):
+            m = ("This buildslave (%s) does not support preserving timestamps. "
+                 "Please upgrade the buildslave." % self.build.slavename )
+            raise BuildSlaveTooOldError(m)
+
         # default arguments
         args = {
             'slavesrc': source,
@@ -281,6 +286,7 @@ class FileUpload(_TransferBuildStep):
             'writer': fileWriter,
             'maxsize': self.maxsize,
             'blocksize': self.blocksize,
+            'keepstamp': self.keepstamp,
             }
 
         self.cmd = StatusRemoteCommand('uploadFile', args)
@@ -288,27 +294,14 @@ class FileUpload(_TransferBuildStep):
         d.addCallback(self.finished).addErrback(self.failed)
 
 
-class DirectoryUpload(BuildStep):
-    """
-    Build step to transfer a directory from the slave to the master.
-
-    arguments:
-
-    - ['slavesrc']   name of source directory at slave, relative to workdir
-    - ['masterdest'] name of destination directory at master
-    - ['workdir']    string with slave working directory relative to builder
-                     base dir, default 'build'
-    - ['maxsize']    maximum size of the compressed tarfile containing the
-                     whole directory
-    - ['blocksize']  maximum size of each block being transfered
-    - ['compress']   compression type to use: one of [None, 'gz', 'bz2']
-
-    """
+class DirectoryUpload(_TransferBuildStep):
 
     name = 'upload'
 
+    renderables = [ 'slavesrc', 'masterdest' ]
+
     def __init__(self, slavesrc, masterdest,
-                 workdir="build", maxsize=None, blocksize=16*1024,
+                 workdir=None, maxsize=None, blocksize=16*1024,
                  compress=None, **buildstep_kwargs):
         BuildStep.__init__(self, **buildstep_kwargs)
         self.addFactoryArguments(slavesrc=slavesrc,
@@ -329,14 +322,13 @@ class DirectoryUpload(BuildStep):
 
     def start(self):
         version = self.slaveVersion("uploadDirectory")
-        properties = self.build.getProperties()
 
         if not version:
             m = "slave is too old, does not know about uploadDirectory"
             raise BuildSlaveTooOldError(m)
 
-        source = properties.render(self.slavesrc)
-        masterdest = properties.render(self.masterdest)
+        source = self.slavesrc
+        masterdest = self.masterdest
         # we rely upon the fact that the buildmaster runs chdir'ed into its
         # basedir to make sure that relative paths in masterdest are expanded
         # properly. TODO: maybe pass the master's basedir all the way down
@@ -353,7 +345,7 @@ class DirectoryUpload(BuildStep):
         # default arguments
         args = {
             'slavesrc': source,
-            'workdir': self.workdir,
+            'workdir': self._getWorkdir(),
             'writer': dirWriter,
             'maxsize': self.maxsize,
             'blocksize': self.blocksize,
@@ -414,29 +406,11 @@ class _FileReader(pb.Referenceable):
 
 
 class FileDownload(_TransferBuildStep):
-    """
-    Download the first 'maxsize' bytes of a file, from the buildmaster to the
-    buildslave. Set the mode of the file
 
-    Arguments::
-
-     ['mastersrc'] filename of source file at master
-     ['slavedest'] filename of destination file at slave
-     ['workdir']   string with slave working directory relative to builder
-                   base dir, default 'build'
-     ['maxsize']   maximum size of the file, default None (=unlimited)
-     ['blocksize'] maximum size of each block being transfered
-     ['mode']      use this to set the access permissions of the resulting
-                   buildslave-side file. This is traditionally an octal
-                   integer, like 0644 to be world-readable (but not
-                   world-writable), or 0600 to only be readable by
-                   the buildslave account, or 0755 to be world-executable.
-                   The default (=None) is to leave it up to the umask of
-                   the buildslave process.
-
-    """
     name = 'download'
 
+    renderables = [ 'mastersrc', 'slavedest' ]
+
     def __init__(self, mastersrc, slavedest,
                  workdir=None, maxsize=None, blocksize=16*1024, mode=None,
                  **buildstep_kwargs):
@@ -458,8 +432,6 @@ class FileDownload(_TransferBuildStep):
         self.mode = mode
 
     def start(self):
-        properties = self.build.getProperties()
-
         version = self.slaveVersion("downloadFile")
         if not version:
             m = "slave is too old, does not know about downloadFile"
@@ -467,8 +439,8 @@ class FileDownload(_TransferBuildStep):
 
         # we are currently in the buildmaster's basedir, so any non-absolute
         # paths will be interpreted relative to that
-        source = os.path.expanduser(properties.render(self.mastersrc))
-        slavedest = properties.render(self.slavedest)
+        source = os.path.expanduser(self.mastersrc)
+        slavedest = self.slavedest
         log.msg("FileDownload started, from master %r to slave %r" %
                 (source, slavedest))
 
@@ -503,28 +475,11 @@ class FileDownload(_TransferBuildStep):
         d.addCallback(self.finished).addErrback(self.failed)
 
 class StringDownload(_TransferBuildStep):
-    """
-    Download the first 'maxsize' bytes of a string, from the buildmaster to the
-    buildslave. Set the mode of the file
 
-    Arguments::
-
-     ['s']         string to transfer
-     ['slavedest'] filename of destination file at slave
-     ['workdir']   string with slave working directory relative to builder
-                   base dir, default 'build'
-     ['maxsize']   maximum size of the file, default None (=unlimited)
-     ['blocksize'] maximum size of each block being transfered
-     ['mode']      use this to set the access permissions of the resulting
-                   buildslave-side file. This is traditionally an octal
-                   integer, like 0644 to be world-readable (but not
-                   world-writable), or 0600 to only be readable by
-                   the buildslave account, or 0755 to be world-executable.
-                   The default (=None) is to leave it up to the umask of
-                   the buildslave process.
-    """
     name = 'string_download'
 
+    renderables = [ 'slavedest', 's' ]
+
     def __init__(self, s, slavedest,
                  workdir=None, maxsize=None, blocksize=16*1024, mode=None,
                  **buildstep_kwargs):
@@ -546,8 +501,6 @@ class StringDownload(_TransferBuildStep):
         self.mode = mode
 
     def start(self):
-        properties = self.build.getProperties()
-
         version = self.slaveVersion("downloadFile")
         if not version:
             m = "slave is too old, does not know about downloadFile"
@@ -555,14 +508,14 @@ class StringDownload(_TransferBuildStep):
 
         # we are currently in the buildmaster's basedir, so any non-absolute
         # paths will be interpreted relative to that
-        slavedest = properties.render(self.slavedest)
+        slavedest = self.slavedest
         log.msg("StringDownload started, from master to slave %r" % slavedest)
 
         self.step_status.setText(['downloading', "to",
                                   os.path.basename(slavedest)])
 
         # setup structures for reading the file
-        fp = StringIO(properties.render(self.s))
+        fp = StringIO(self.s)
         fileReader = _FileReader(fp)
 
         # default arguments
@@ -580,14 +533,9 @@ class StringDownload(_TransferBuildStep):
         d.addCallback(self.finished).addErrback(self.failed)
 
 class JSONStringDownload(StringDownload):
-    """
-    Encode object o as a json string and save it on the buildslave
 
-    Arguments::
-
-     ['o']         object to encode and transfer
-    """
     name = "json_download"
+
     def __init__(self, o, slavedest, **buildstep_kwargs):
         if 's' in buildstep_kwargs:
             del buildstep_kwargs['s']
@@ -596,11 +544,9 @@ class JSONStringDownload(StringDownload):
         self.addFactoryArguments(o=o)
 
 class JSONPropertiesDownload(StringDownload):
-    """
-    Download the current build properties as a json string and save it on the
-    buildslave
-    """
+
     name = "json_properties_download"
+
     def __init__(self, slavedest, **buildstep_kwargs):
         self.super_class = StringDownload
         if 's' in buildstep_kwargs:
